package org.mmga.clubs.controller;

import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONException;
import com.alibaba.fastjson2.JSONObject;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.annotation.Nullable;
import jakarta.websocket.OnClose;
import jakarta.websocket.OnMessage;
import jakarta.websocket.OnOpen;
import jakarta.websocket.Session;
import jakarta.websocket.server.PathParam;
import jakarta.websocket.server.ServerEndpoint;
import lombok.NoArgsConstructor;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.mmga.clubs.entities.chess.Room;
import org.mmga.clubs.entities.chess.packet.*;
import org.mmga.clubs.entities.chess.packet.request.*;
import org.mmga.clubs.entities.user.User;
import org.mmga.clubs.service.UserService;
import org.mmga.clubs.utils.JwtUtils;
import org.mmga.clubs.utils.WebSocketUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.lang.reflect.Field;
import java.util.Collection;
import java.util.Optional;
import java.util.UUID;
import java.util.concurrent.ConcurrentHashMap;

@Slf4j
@Tag(name = "五子棋", description = "五子棋相关websocket接口")
@ServerEndpoint("/chess/{token}")
@Component
@NoArgsConstructor
public class ChessController {
    private static final ConcurrentHashMap<String, Session> sessions = new ConcurrentHashMap<>();
    private static final ConcurrentHashMap<String, User> users = new ConcurrentHashMap<>();
    private static final ConcurrentHashMap<UUID, Room> rooms = new ConcurrentHashMap<>();
    private static JwtUtils jwtUtils;
    private static UserService userService;

    @Autowired
    public void setJwtUtils(JwtUtils jwtUtils) {
        ChessController.jwtUtils = jwtUtils;
    }

    @Autowired
    public void setUserService(UserService userService) {
        ChessController.userService = userService;
    }

    @SneakyThrows
    @OnOpen
    public void onOpen(Session session, @PathParam("token") String token) {
        Optional<Integer> i = jwtUtils.verifyTokenSafe(token);
        if (i.isEmpty()) {
            WebSocketUtils.sendPacket(new ErrorPacket("token验证失败！"), session);
            session.close();
            return;
        }
        String sessionId = session.getId();
        User user = userService.getUserById(i.get());
        WebSocketUtils.sendPacket(new UserInfoPacket(user), session);
        users.put(sessionId, user);
        sessions.put(sessionId, session);
    }

    @OnMessage
    public void onMessage(String message, Session session) {
        try {
            JSONObject jsonObject = JSON.parseObject(message);
            Object response = handleMessage(jsonObject, session);
            if (response != null) {
                WebSocketUtils.sendPacket(response, session);
            }
        } catch (JSONException e) {
            WebSocketUtils.sendPacket(new ErrorPacket("JSON格式错误"), session);
        }

    }

    @Nullable
    public Object handleMessage(JSONObject object, Session session) {
        String packetName = object.getString("name");
        if (packetName == null) return new ErrorPacket("错误的请求类型");
        try {
            Class<?> packetType = Class.forName("org.mmga.clubs.entities.chess.packet.request." + packetName + "Request");
            JSONObject payloadJson = object.getJSONObject("payload");
            if (payloadJson == null) {
                Field[] declaredFields = packetType.getDeclaredFields();
                if (declaredFields.length == 0) {
                    payloadJson = new JSONObject();
                }
            }
            if (payloadJson == null) return new ErrorPacket("请求类型参数错误！");
            Object payload = payloadJson.to(packetType);
            String sessionId = session.getId();
            if (payload instanceof PlayerJoinRequest joinRequest) {
                Room roomByPlayer = getRoomByPlayer(sessionId);
                if (roomByPlayer != null) return new ErrorPacket("你已经加入过一个房间了");
                Room room = rooms.get(joinRequest.roomId());
                if (room == null) return new ErrorPacket("未知的房间");
                Optional<String> join = room.join(session, users.get(sessionId));
                if (join.isPresent()) return new ErrorPacket(join.get());
                room.broadcast(room.getRoomInfo(users));
            }
            if (payload instanceof RoomListRequest) {
                Collection<Room> values = rooms.values();
                return new RoomListPacket(values);
            }
            if (payload instanceof CreateRoomRequest) {
                Room room = new Room();
                UUID roomId = room.getId();
                rooms.put(roomId, room);
                return new RoomCreatedPacket(roomId);
            }
            if (payload instanceof PlaceChessPieceRequest placeChessPieceRequest) {
                int x = placeChessPieceRequest.x();
                int y = placeChessPieceRequest.y();
                Room roomByPlayer = getRoomByPlayer(sessionId);
                if (roomByPlayer == null) return new ErrorPacket("你还没加入房间呢");
                Optional<String> s = roomByPlayer.downPiece(x, y, sessionId);
                if (s.isPresent()) return new ErrorPacket(s.get());
                roomByPlayer.broadcast(roomByPlayer.getRoomInfo(users));
                Optional<Room.WinInfo> winInfo = roomByPlayer.isWin();
                winInfo.ifPresent(info -> {
                    roomByPlayer.broadcast(new HasPlayerWinPacket(info));
                    if (info.isWhite()) {
                        WebSocketUtils.sendPacketIfPossible(new PlayerWinPacket(), roomByPlayer.getWhiteSession());
                        WebSocketUtils.sendPacketIfPossible(new PlayerLosePacket(), roomByPlayer.getBlackSession());
                    } else {
                        WebSocketUtils.sendPacketIfPossible(new PlayerWinPacket(), roomByPlayer.getBlackSession());
                        WebSocketUtils.sendPacketIfPossible(new PlayerLosePacket(), roomByPlayer.getWhiteSession());
                    }
                });
            }
            if (payload instanceof ResetRoomRequest) {
                Room roomByPlayer = getRoomByPlayer(sessionId);
                if (roomByPlayer == null) return new ErrorPacket("你还没加入房间呢");
                roomByPlayer.requestRestart(session);
                roomByPlayer.broadcast(roomByPlayer.getRoomInfo(users));
            }
        } catch (ClassNotFoundException e) {
            return new ErrorPacket("错误的请求类型");
        }
        return null;
    }

    @Nullable
    private Room getRoomByPlayer(String id) {
        for (Room value : rooms.values()) {
            if (value.isIn(id)) return value;
        }
        return null;
    }

    @OnClose
    public void onClose(Session session) {
        String sessionId = session.getId();
        Room roomByPlayer = this.getRoomByPlayer(sessionId);
        if (roomByPlayer != null) {
            roomByPlayer.leave(session);
            roomByPlayer.broadcast(roomByPlayer.getRoomInfo(users));
        }
        log.info("session closed {}", sessionId);
        sessions.remove(sessionId);
        users.remove(sessionId);
    }
}
